/*
 * CALLBACK.C
 *
 * An "owner draw" sprite. This example writes the number of "hits" between
 * the sprite and the window edges in the sprite itself.
 *
 * This example shows:
 * o  using a DDB callback function
 * o  setting "user data" for a sprite
 *
 * Compiling:
 *   Borland C++, 16-bit
 *      bcc -W callback.c as16.lib
 *
 *   Borland C++, 32-bit
 *      bcc32 -tW callback.c as32b.lib
 *
 *   Microsoft C/C++, 16 bit
 *      cl -Gw callback.c as16.lib slibcew.lib libw.lib example.def
 *      rc example.rc callback.exe
 *
 *   Microsoft C/C++, 32 bit
 *      cl -GA callback.c as32m.lib user32.lib gdi32.lib
 *
 *   Watcom C/C++ 11.0, 32-bit
 *      wcl386 /l=nt_win callback.c as32w.lib
 *
 * Copyright (c) 1997-1999, ITB CompuPhase. You may use/modify/distribute
 * this file or portions of it. It is provided as an example for the use of
 * the AniSprite API. There are no waranties on the correct behavior of this
 * program.
 */
#include <stdlib.h>
#include <windows.h>
#include "..\include\anispri.h"

#define BOARD_WIDTH     640
#define BOARD_HEIGHT    480
#define TIMER_INTERVAL  50

static HINSTANCE hInst;

LRESULT CALLBACK AnimWinFunc(HWND hwnd, unsigned message,
                             WPARAM wParam, LPARAM lParam);


int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{
  MSG msg;
  WNDCLASS wc;
  RECT rect;
  DWORD dwStyle;

  wc.style = 0;
  wc.lpfnWndProc = (WNDPROC)AnimWinFunc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon(hInstance, (LPSTR)"anim_icon");
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = GetStockObject(LTGRAY_BRUSH);
  wc.lpszMenuName = (LPSTR)NULL;
  wc.lpszClassName = "Animation";
  if (!RegisterClass(&wc))
    return FALSE;

  hInst = hInstance;

  /* creat a window with the right size for the board */
  SetRect(&rect, 0, 0, BOARD_WIDTH, BOARD_HEIGHT);
  dwStyle = WS_POPUPWINDOW | WS_CAPTION | WS_VISIBLE;
  AdjustWindowRect(&rect, dwStyle, FALSE);
  CreateWindow("Animation", "AniSprite: Callback", dwStyle,
               50, 50, rect.right - rect.left, rect.bottom - rect.top,
               0, 0, hInstance, NULL);

  while (GetMessage(&msg, NULL, 0, 0)) {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  } /* while */

  return msg.wParam;
}

DWORD FAR PASCAL SpriteCallback(ASPRITE Sprite, int Event,
                                DWORD Value, LPVOID Data)
{
  ASPAINT_DDB aspaint = (ASPAINT_DDB)Data;
  long hits = (long)as_GetData(Sprite, AS_DATA_USERDATA, 0);
  char str[10];

  if (Event == AS_EVENT_PAINT_DDB) {
    wsprintf(str, "%ld", hits);
    SetBkMode(aspaint->hDC, TRANSPARENT);
    SetTextColor(aspaint->hDC, RGB(0,0,0));
    TextOut(aspaint->hDC, 10, 24, str, lstrlen(str));
  } /* if */
  return 0L;
}

LRESULT CALLBACK AnimWinFunc(HWND hwnd, unsigned message,
                             WPARAM wParam, LPARAM lParam)
{
static ASBOARD Board;
static ASPRITE Sprite;
static int stepx, stepy;
  PAINTSTRUCT ps;
  HDC hdc;
  int x, y, width, height;
  long hits;
  FARPROC lpfn;

  switch (message) {
  case WM_CREATE:
    /* load the board (with an identity palette) and the sprite */
    Board = as_LoadBoard("maze.bmp", AS_MODE_PAL_COLORS, NULL, NULL);
    Sprite = as_Load("sprite2.bmp", PALETTERGB(0xff, 0x00, 0xff), NULL, NULL);

    /* assign and show the sprite */
    as_Assign(Sprite, Board, 0);
    as_Show(Sprite, TRUE);

    /* install the callback */
    lpfn = MakeProcInstance((FARPROC)SpriteCallback, hInst);
    as_SetData(Sprite, AS_DATA_CALLBACK, AS_EVENT_PAINT_DDB, (LPVOID)lpfn);

    /* store the number of "hits" in the sprite's user data */
    as_SetData(Sprite, AS_DATA_USERDATA, 0, (LPVOID)0);

    /* Create a timer to move the sprite, set initial direction */
    as_SetTimer(Board, hwnd, TIMER_INTERVAL);
    stepx = 5;
    stepy = 3;
    hits = 0;
    break;

  case WM_DESTROY:
    /* remove the callback */
    lpfn = as_GetData(Sprite, AS_DATA_CALLBACK, AS_EVENT_PAINT_DDB);
    FreeProcInstance(lpfn);
    as_Assign(Sprite, Board, -1);
    as_Delete(Sprite, TRUE);
    as_DeleteBoard(Board, TRUE);
    PostQuitMessage(0);
    break;

  case AS_TIMER:
    /* get current position */
    x = as_GetValue(Sprite, AS_VALUE_XPOS);
    y = as_GetValue(Sprite, AS_VALUE_YPOS);
    width = as_GetValue(Sprite,AS_VALUE_WIDTH);
    height = as_GetValue(Sprite,AS_VALUE_HEIGHT);
    hits = (long)as_GetData(Sprite, AS_DATA_USERDATA, 0);

    /* Move the image around a bit, check against borders */
    x += stepx;
    y += stepy;
    if (x < 0) {
      x = 0;
      stepx = -stepx;
      hits++;
    } else if (x + width > BOARD_WIDTH) {
      x = BOARD_WIDTH - width;
      stepx = -stepx;
      hits++;
    } /* if */
    if (y < 0) {
      y = 0;
      stepy = -stepy;
      hits++;
    } else if (y + height > BOARD_HEIGHT) {
      y = BOARD_HEIGHT - height;
      stepy = -stepy;
      hits++;
    } /* if */

    as_SetData(Sprite, AS_DATA_USERDATA, 0, (LPVOID)hits);
    as_SetPos(Sprite, x, y);

    /* repaint the board */
    hdc = GetDC(hwnd);
    as_PaintBoard(hdc, Board, 0, 0, FALSE);
    ReleaseDC(hwnd, hdc);
    break;

  case WM_PAINT:
    hdc=BeginPaint(hwnd, &ps);
    as_PaintBoard(hdc, Board, 0, 0, TRUE);
    EndPaint(hwnd, &ps);
    break;

  case WM_ERASEBKGND:
  case WM_PALETTECHANGED:
  case WM_QUERYNEWPALETTE:
    return as_ForwardMessage(Board, hwnd, message, wParam, lParam);

  default:
    return DefWindowProc(hwnd, message, wParam, lParam);
  } /* switch */

  return 0L;
}

