/*
 * GHOST.C
 *
 * Alpha blending (a translucent sprite).
 *
 * This example shows:
 * o  Changing a cutout mask to an alpha mask.
 * o  Preparing the board for alpha masks and creating a alpha levels.
 *
 * Compiling:
 *   Borland C++, 16-bit
 *      bcc -W ghost.c ..\lib\as16.lib
 *
 *   Borland C++, 32-bit
 *      bcc32 -tW ghost.c ..\lib\as32b.lib
 *
 *   Microsoft C/C++, 16 bit
 *      cl -Gw ghost.c ..\lib\as16.lib slibcew.lib libw.lib example.def
 *      rc example.rc ghost.exe
 *
 *   Microsoft C/C++, 32 bit
 *      cl -GA ghost.c ..\lib\as32m.lib user32.lib gdi32.lib
 *
 *   Watcom C/C++ 11.0, 32-bit
 *      wcl386 /l=nt_win ghost.c ..\lib\as32w.lib
 *
 *   LCC-Win32
 *      lcc ghost.c
 *      lcclnk ghost.obj ..\lib\as32l.lib
 *
 * Copyright (c) 1997-1999, ITB CompuPhase. You may use/modify/distribute
 * this file or portions of it. It is provided as an example for the use of
 * the AniSprite API. There are no waranties on the correct behavior of this
 * program.
 */
#include <windows.h>
#include "..\include\anispri.h"

#include "alphatbl.c"   /* code to read/write alpha tables */

#define BOARD_WIDTH     640
#define BOARD_HEIGHT    480
#define ID_TIMER        1
#define TIMER_INTERVAL  50
#define U_CREATE        WM_USER


LRESULT CALLBACK AnimWinFunc(HWND hwnd, unsigned message,
                             WPARAM wParam, LPARAM lParam);


int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{
  MSG msg;
  WNDCLASS wc;
  RECT rect;
  DWORD dwStyle;
  HWND hwnd;

  wc.style = 0;
  wc.lpfnWndProc = (WNDPROC)AnimWinFunc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon(hInstance, (LPSTR)"anim_icon");
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = GetStockObject(LTGRAY_BRUSH);
  wc.lpszMenuName = (LPSTR)NULL;
  wc.lpszClassName = "Animation";
  if (!RegisterClass(&wc))
    return FALSE;

  /* creat a window with the right size for the board */
  SetRect(&rect, 0, 0, BOARD_WIDTH, BOARD_HEIGHT);
  dwStyle = WS_POPUPWINDOW | WS_CAPTION | WS_VISIBLE;
  AdjustWindowRect(&rect, dwStyle, FALSE);
  hwnd = CreateWindow("Animation", "AniSprite: Translucent sprites", dwStyle,
                      50, 50, rect.right - rect.left, rect.bottom - rect.top,
                      0, 0, hInstance, NULL);
  UpdateWindow(hwnd);

  while (GetMessage(&msg, NULL, 0, 0)) {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  } /* while */

  return msg.wParam;
}

static void move(int *x, int *y, int *stepx, int *stepy, int width, int height)
{
  *x += *stepx;
  *y += *stepy;
  if (*x < 0) {
    *x = 0;
    *stepx = - *stepx;
  } else if (*x + width > BOARD_WIDTH) {
    *x = BOARD_WIDTH - width;
    *stepx = - *stepx;
  } /* if */
  if (*y < 0) {
    *y = 0;
    *stepy = - *stepy;
  } else if (*y + height > BOARD_HEIGHT) {
    *y = BOARD_HEIGHT - height;
    *stepy = - *stepy;
  } /* if */
}

LRESULT CALLBACK AnimWinFunc(HWND hwnd, unsigned message,
                             WPARAM wParam, LPARAM lParam)
{
static COLORREF crTransColor[] = { PALETTEINDEX(253) };
static ASBOARD Board;
static ASPRITE Sprite1, Sprite2;
static int step1x, step1y;      /* for first sprite */
static int step2x, step2y;      /* for second sprite */
static char szWaitMessage[] = "Calculating alpha levels, one moment please...";
static BOOL ReadFromFile;       /* if TRUE, alpha table was loaded from a file */
  LPBITMAPINFO lpBitsInfo;
  LPVOID lpBits, lpMask;
  PAINTSTRUCT ps;
  RECT rect;
  HDC hdc;
  int x, y, width, height;
  LPVOID lpAlpha;
  #if defined(__WIN32__)
    SIZE size;
  #else
    DWORD dwSize;
  #endif

  switch (message) {
  case WM_CREATE:
    /* lpBitsInfo is allocated temporarily */
    lpBitsInfo = as_AllocResource(sizeof(BITMAPINFO) + 256 * sizeof(RGBQUAD));

    /* create the board (with an identity palette) */
    lpBits = as_LoadDIB("map.bmp", lpBitsInfo, NULL, NULL);
    Board = as_CreateBoard(lpBitsInfo, lpBits, NULL,
                           AS_MODE_PAL_COLORS | AS_MODE_ALPHA);
    as_SetPalette(Board, lpBitsInfo->bmiColors, TRUE);

    /* already repaint the screen (because the next step takes
     * a while), then continue with the remainder of the
     * initialization */
    PostMessage(hwnd, U_CREATE, 0, (DWORD)lpBitsInfo);
    break;

  case U_CREATE:
    lpBitsInfo = (LPBITMAPINFO)lParam;

    ReadFromFile = LoadAlphaLevels(Board, "ghost.ast", NULL);
    if (!ReadFromFile) {
      /* calculate and add one alpha levels, at 67% opaqueness */
      lpAlpha = as_CreateAlphaLevel(lpBitsInfo->bmiColors, 85);
      as_SetBoardData(Board, AS_DATA_ALPHA_LEVEL, 1, lpAlpha);
    } /* if */

    /* create the mask, and map the "opaque" portion of the mask to
     * the alpha level that was created earlier */
    lpBits = as_LoadDIB("ghost1.bmp", lpBitsInfo, NULL, NULL);
    lpMask = as_CreateMask(lpBitsInfo, lpBits, TRUE, crTransColor, 1);
    as_MapMaskLevel((int)lpBitsInfo->bmiHeader.biWidth,
                    (int)lpBitsInfo->bmiHeader.biHeight,
                    lpMask, 0, 1);

    /* create the sprite */
    Sprite1 = as_Create((int)lpBitsInfo->bmiHeader.biWidth,
                       (int)lpBitsInfo->bmiHeader.biHeight,
                       lpMask, AS_MASK_ALPHA, lpBits, TRUE);

    /* same for the other sprite */
    lpBits = as_LoadDIB("ghost2.bmp", lpBitsInfo, NULL, NULL);
    lpMask = as_CreateMask(lpBitsInfo, lpBits, TRUE, crTransColor, 1);
    as_MapMaskLevel((int)lpBitsInfo->bmiHeader.biWidth,
                    (int)lpBitsInfo->bmiHeader.biHeight,
                    lpMask, 0, 1);
    Sprite2 = as_Create((int)lpBitsInfo->bmiHeader.biWidth,
                       (int)lpBitsInfo->bmiHeader.biHeight,
                       lpMask, AS_MASK_ALPHA, lpBits, TRUE);

    /* assign and show the sprites */
    as_Assign(Sprite1, Board, 0);
    as_Assign(Sprite2, Board, 0);
    as_Show(Sprite1, TRUE);
    as_Show(Sprite2, TRUE);

    /* clean up */
    as_FreeResource(lpBitsInfo);
    InvalidateRect(hwnd, NULL, FALSE);

    /* Create a timer to move the sprite, set initial direction */
    as_SetTimer(Board, hwnd, TIMER_INTERVAL);
    step1x = 5;
    step1y = 3;
    step2x = 3;
    step2y = 5;
    break;

  case WM_DESTROY:
    if (!ReadFromFile
        && MessageBox(hwnd, "Save the alpha table?","AniSprite", MB_YESNO) == IDYES)
      SaveAlphaLevels(Board, "ghost.ast", 1);
    /* the sprites */
    as_Assign(Sprite1, Board, -1);
    as_Assign(Sprite2, Board, -1);
    as_Delete(Sprite1, TRUE);
    as_Delete(Sprite2, TRUE);
    /* the alpha level */
    lpAlpha = as_GetBoardData(Board, AS_DATA_ALPHA_LEVEL, 1);
    as_FreeResource(lpAlpha);
    /* the board */
    as_DeleteBoard(Board, TRUE);
    /* the rest */
    PostQuitMessage(0);
    break;

  case AS_TIMER:
    /* get current position */
    x = as_GetValue(Sprite1, AS_VALUE_XPOS);
    y = as_GetValue(Sprite1, AS_VALUE_YPOS);
    width = as_GetValue(Sprite1,AS_VALUE_WIDTH);
    height = as_GetValue(Sprite1,AS_VALUE_HEIGHT);

    /* move the image around a bit, check against borders */
    move(&x, &y, &step1x, &step1y, width, height);
    as_SetPos(Sprite1, x, y);

    /* same for the other sprite */
    x = as_GetValue(Sprite2, AS_VALUE_XPOS);
    y = as_GetValue(Sprite2, AS_VALUE_YPOS);
    width = as_GetValue(Sprite1,AS_VALUE_WIDTH);
    height = as_GetValue(Sprite1,AS_VALUE_HEIGHT);
    move(&x, &y, &step2x, &step2y, width, height);
    as_SetPos(Sprite2, x, y);

    /* repaint the board */
    hdc = GetDC(hwnd);
    as_PaintBoard(hdc, Board, 0, 0, FALSE);
    ReleaseDC(hwnd, hdc);
    break;

  case WM_PAINT:
    hdc=BeginPaint(hwnd, &ps);
    as_PaintBoard(hdc, Board, 0, 0, TRUE);
    /* Display a "banner" in the screen, because calculating
     * an alpha level is a lengthy process.
     */
    if (step1x == 0 && step1y == 0) {
      GetClientRect(hwnd, &rect);
      #if defined(__WIN32__)
        GetTextExtentPoint(hdc, szWaitMessage, lstrlen(szWaitMessage), &size);
        width = size.cx;
        height = size.cy;
      #else
        dwSize = GetTextExtent(hdc, szWaitMessage, lstrlen(szWaitMessage));
        width = LOWORD(dwSize);
        height = HIWORD(dwSize);
      #endif
      rect.bottom = rect.top + height + 5;
      FillRect(hdc, &rect, GetStockObject(LTGRAY_BRUSH));
      SetTextColor(hdc, RGB(255, 0, 0));
      SetBkMode(hdc, TRANSPARENT);
      TextOut(hdc, (rect.right-width)/2, 2, szWaitMessage,
              lstrlen(szWaitMessage));
    } /* if */
    EndPaint(hwnd, &ps);
    break;

  case WM_ERASEBKGND:
  case WM_PALETTECHANGED:
  case WM_QUERYNEWPALETTE:
    return as_ForwardMessage(Board, hwnd, message, wParam, lParam);

  default:
    return DefWindowProc(hwnd, message, wParam, lParam);
  } /* switch */

  return 0L;
}

